package com.fzm.common.service.impl;

import cn.fzm.chain.simplesdk.client.ParaChainClient;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.RandomUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fzm.common.constant.SystemConstant;
import com.fzm.common.entity.AuthPerson;
import com.fzm.common.entity.User;
import com.fzm.common.entity.vo.UserListVo;
import com.fzm.common.entity.vo.UserStatisticVo;
import com.fzm.common.enums.AuthStatusEnum;
import com.fzm.common.enums.AuthTypeEnum;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.mapper.UserMapper;
import com.fzm.common.params.LoginParam;
import com.fzm.common.properties.OssProperties;
import com.fzm.common.properties.SmsProperties;
import com.fzm.common.service.AuthPersonService;
import com.fzm.common.service.UserService;
import com.fzm.common.utils.JwtUtil;
import com.fzm.common.utils.OssUtil;
import com.fzm.common.utils.RedisUtil;
import com.fzm.common.utils.SmsUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * @author tangtuo
 * @date 2021/6/23 15:15
 */
@Service
@Transactional(rollbackFor = RuntimeException.class)
public class UserServiceImpl extends ServiceImpl<UserMapper, User> implements UserService {

    @Resource
    private UserMapper userMapper;

    @Resource
    private PasswordEncoder passwordEncoder;

    @Resource
    private SmsProperties smsProperties;

    @Resource
    private SmsUtil smsUtil;

    @Resource
    private RedisUtil redisUtil;

    @Resource
    private AuthPersonService authPersonService;

    @Resource
    private ParaChainClient paraClient;

    @Resource
    private HttpServletRequest request;

    @Resource
    private OssUtil ossUtil;

    @Resource
    private OssProperties ossProperties;

    @Override
    public User loadUserByUsername(String username) {
        QueryWrapper<User> wrapper = new QueryWrapper<>();
        wrapper.eq("telephone", username).
                or().
                eq("email", username);
        return getOne(wrapper);
    }


    @Override
    public User login(LoginParam param) {
        String username = StringUtils.isNotBlank(param.getTelephone()) ?
                param.getTelephone() :
                param.getEmail();
        User user = loadUserByUsername(username);
        if (user == null) {
            // 当前手机号或者邮箱没有注册过,走注册逻辑
            User registerUser = new User();
            BeanUtil.copyProperties(param, registerUser, true);
            // 生成钱包
            String wallet = paraClient.walletCreateAccount(IdUtil.simpleUUID());
            if (StringUtils.isBlank(wallet)) {
                throw GlobalException.newException(ResultCode.UNAUTHORIZED, "注册失败,生成区块链钱包异常,请稍后再试");
            }
            registerUser.setWallet(wallet);
            // 用户注册时生成默认昵称
            String nickname = "INMVO-" + RandomUtil.randomInt(100000, 999999);
            registerUser.setNickname(nickname);
            if (StringUtils.isNotBlank(param.getPassword())) {
                registerUser.setPassword(passwordEncoder.encode(param.getPassword()));
            } else {
                // 验证码注册流程
                if (!smsUtil.validateCode(getLoginCodeType(param.getCodetype()), username, param.getVerificationCode(), param.getCodetype())) {
                    throw GlobalException.newException(ResultCode.VALIDATE_FAILED, "验证码校验失败");
                }
            }
            save(registerUser);
            redisUtil.delete("user::statistic");
            return getById(registerUser.getId());
        } else {
            // 登录逻辑
            if (StringUtils.isNotBlank(param.getPassword())) {
                // 密码登录流程
                if (!passwordEncoder.matches(param.getPassword(), user.getPassword())) {
                    throw GlobalException.newException(ResultCode.LOGIN_ERROR, "密码有误");
                }
            } else {
                // 验证码登录流程
                if (!smsUtil.validateCode(getLoginCodeType(param.getCodetype()), username, param.getVerificationCode(), param.getCodetype())) {
                    throw GlobalException.newException(ResultCode.VALIDATE_FAILED, "验证码校验失败");
                }
            }
            return user;
        }
    }

    @Override
    public User getUserByToken() {
        String token = request.getHeader("Authorization");
        if (StringUtils.isBlank(token)) {
            throw GlobalException.newException(ResultCode.UNAUTHORIZED, "您还没有登录,请先登录");
        }
        Integer userId = JwtUtil.getUserIdFromToken(token);
        return getById(userId);
    }


    public String getLoginCodeType(String type) {
        switch (type) {
            case "sms":
                return smsProperties.getLoginMessageCodetype();
            case "email":
                return smsProperties.getLoginMailCodetype();
            case "voice":
                return smsProperties.getLoginVoiceCodetype();
            default:
                throw GlobalException.newException(ResultCode.VALIDATE_FAILED, "短信类型【" + type + "】的传参有误,取值范围是{sms,email,voice}");
        }
    }

    @Override
    public Boolean personAuth(AuthPerson authPerson) {
        Integer loginId = JwtUtil.getUserIdFromToken(request.getHeader("Authorization"));
        User user = new User().
                setAuthStatus(AuthStatusEnum.ONGOING.getStatus()).
                setAuthType(AuthTypeEnum.PERSON.getType()).
                setId(loginId);
        updateById(user);
        authPerson.setUserId(loginId);
        return authPersonService.save(authPerson);
    }

    @Override
    public User uploadAvatar(String avatar) {
        User loginUser = getUserByToken();
        // 如果不是第一次上传头像,需要删除旧头像
        if (StringUtils.isNotBlank(loginUser.getAvatar())) {
            ossUtil.delete(loginUser.getAvatar(), ossProperties.getSimpleBucket());
        }
        User user = new User().
                setAvatar(avatar).
                setId(loginUser.getId());
        updateById(user);
        return getById(loginUser.getId());
    }

    @Override
    @Cacheable(value = "user", key = "'statistic'")
    public UserStatisticVo getStatistic() {
        UserStatisticVo statisticVo = new UserStatisticVo();
        List<User> list = list();
        // 已注册用户数
        statisticVo.setRegisterUserCount(list.size());
        // 已发行NFT用户数
        long publishUserCount = list.stream().filter(user -> SystemConstant.BOOLEAN_DATA_TRUE.equals(user.getIsPublish())).count();
        statisticVo.setPublishUserCount((int) publishUserCount);
        // 已实名用户数
        long authUserCount = list.stream().filter(user -> AuthStatusEnum.SUCCESS.getStatus().equals(user.getAuthStatus())).count();
        statisticVo.setAuthUserCount((int) authUserCount);
        // 个人实名用户数
        long personAuthCount = list.stream().
                filter(user -> AuthTypeEnum.PERSON.getType().equals(user.getAuthType()) && AuthStatusEnum.SUCCESS.getStatus().equals(user.getAuthStatus())).
                count();
        statisticVo.setPersonAuthCount((int) personAuthCount);
        // 企业实名用户数
        long enterpriseAuthCount = list.stream().
                filter(user -> AuthTypeEnum.ENTERPRISE.getType().equals(user.getAuthType()) && AuthStatusEnum.SUCCESS.getStatus().equals(user.getAuthStatus())).
                count();
        statisticVo.setEnterpriseAuthCount((int) enterpriseAuthCount);
        return statisticVo;
    }

    @Override
    public User getUserByWallet(String publishAddress) {
        QueryWrapper<User> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("wallet", publishAddress);
        return getOne(queryWrapper);
    }

    @Override
    public List<UserListVo> list(String telephone, String nickname, Integer authType, String start, String end) {
        DateTime startDate = null;
        DateTime endDate = null;
        if (StringUtils.isNotBlank(start)) {
            startDate = DateUtil.parse(start + " 00:00:00", "yyyy-MM-dd HH:mm:ss");
        }
        if (StringUtils.isNotBlank(end)) {
            endDate = DateUtil.parse(end + " 23:59:59", "yyyy-MM-dd HH:mm:ss");
        }
        return userMapper.list(telephone, nickname, authType, startDate, endDate);
    }
}
