package com.fzm.common.utils;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.fzm.common.constant.TokenConstant;
import com.fzm.common.entity.AbstractUser;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import io.jsonwebtoken.Claims;
import io.jsonwebtoken.ExpiredJwtException;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;
import lombok.extern.slf4j.Slf4j;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @author tangtuo
 * @date 2021/6/23 9:40
 */
@Slf4j
public class JwtUtil {

    private static final String CLAIM_KEY_USERNAME = "sub";
    private static final String CLAIM_KEY_CREATED = "created";
    private static final String CLAIM_KEY_USERID = "userId";
    private static final String CLAIM_KEY_APP_ID = "appId";
    private static final String SECRET = "fzm-ly";
    private static final Long PORTAL_EXPIRATION = 1000 * 60 * 60 * 24L;
    private static final Long ADMIN_EXPIRATION = 1000 * 60 * 60 * 2L;
    private static final String TOKEN_PREFIX = "Bearer ";

    /**
     * 根据负责生成JWT的token
     */
    private static String generateToken(Map<String, Object> claims) {
        return Jwts.builder()
                .setClaims(claims)
                .setExpiration(generateExpirationDate((String) claims.get(CLAIM_KEY_APP_ID)))
                .signWith(SignatureAlgorithm.HS512, SECRET)
                .compact();
    }

    /**
     * 从token中获取JWT中的负载
     */
    private static Claims getClaimsFromToken(String token) {
        Claims claims = null;
        token = token.replace(TOKEN_PREFIX, "");
        try {
            claims = Jwts.parser()
                    .setSigningKey(SECRET)
                    .parseClaimsJws(token)
                    .getBody();
        } catch (ExpiredJwtException e) {
            throw GlobalException.newException(ResultCode.UNAUTHORIZED, "登录已过期");
        } catch (Exception e) {
            throw GlobalException.newException(ResultCode.UNAUTHORIZED, e.getMessage());
        }
        return claims;
    }

    /**
     * 生成token的过期时间
     */
    private static Date generateExpirationDate(String appId) {
        long expiration;
        if (TokenConstant.TOKEN_APP_ID_PORTAL.equals(appId)) {
            expiration = PORTAL_EXPIRATION;
        } else {
            expiration = ADMIN_EXPIRATION;
        }
        return new Date(System.currentTimeMillis() + expiration);
    }

    /**
     * 从token中获取登录用户名
     */
    public static Date getCreateTimeFromToken(String token) {
        try {
            Claims claims = getClaimsFromToken(token);
            return claims.get(CLAIM_KEY_CREATED, Date.class);
        } catch (Exception e) {
            throw GlobalException.newException(ResultCode.UNAUTHORIZED, e.getMessage());
        }
    }

    /**
     * 从token中获取登录用户id
     */
    public static Integer getUserIdFromToken(String token) {
        try {
            Claims claims = getClaimsFromToken(token);
            return claims.get(CLAIM_KEY_USERID, Integer.class);
        } catch (Exception e) {
            throw GlobalException.newException(ResultCode.UNAUTHORIZED, e.getMessage());
        }
    }

    /**
     * 从token中获取登录appId
     */
    public static String getAppIdFromToken(String token) {
        try {
            Claims claims = getClaimsFromToken(token);
            return claims.get(CLAIM_KEY_APP_ID, String.class);
        } catch (Exception e) {
            throw GlobalException.newException(ResultCode.UNAUTHORIZED, e.getMessage());
        }
    }

    /**
     * 验证token是否还有效
     *
     * @param token       客户端传入的token
     * @param userDetails 从数据库中查询出来的用户信息
     */
/*    public static boolean validateToken(String token, UserDetails userDetails) {
        String username = getUserNameFromToken(token);
        return username.equals(userDetails.getUsername()) && !isTokenExpired(token);
    }*/

    /**
     * 判断token是否已经失效
     */
    public static boolean isTokenExpired(String token) {
        Date expiredDate = getExpiredDateFromToken(token);
        return expiredDate.before(new Date());
    }

    /**
     * 从token中获取过期时间
     */
    private static Date getExpiredDateFromToken(String token) {
        Claims claims = getClaimsFromToken(token);
        return claims.getExpiration();
    }


    /**
     * 根据用户信息生成token
     */
    public static String generateToken(AbstractUser userDetails) {
        Map<String, Object> claims = new HashMap<>();
        //claims.put(CLAIM_KEY_USERNAME, userDetails.getUsername());
        claims.put(CLAIM_KEY_CREATED, new Date());
        claims.put(CLAIM_KEY_APP_ID, userDetails.appId());
        claims.put(CLAIM_KEY_USERID, userDetails.userId());
        return generateToken(claims);
    }

    /**
     * 当原来的token没过期时是可以刷新的
     *
     * @param oldToken 带tokenHead的token
     */
    public static String refreshHeadToken(String oldToken) {
        if (StrUtil.isEmpty(oldToken)) {
            return null;
        }
        String token = oldToken.substring(TOKEN_PREFIX.length());
        if (StrUtil.isEmpty(token)) {
            return null;
        }
        //token校验不通过
        Claims claims = getClaimsFromToken(token);
        if (claims == null) {
            return null;
        }
        //如果token已经过期，不支持刷新
        if (isTokenExpired(token)) {
            return null;
        }
        //如果token在30分钟之内刚刷新过，返回原token
        if (tokenRefreshJustBefore(token, 30 * 60)) {
            return token;
        } else {
            claims.put(CLAIM_KEY_CREATED, new Date());
            return generateToken(claims);
        }
    }

    /**
     * 判断token在指定时间内是否刚刚刷新过
     *
     * @param token 原token
     * @param time  指定时间（秒）
     */
    private static boolean tokenRefreshJustBefore(String token, int time) {
        Claims claims = getClaimsFromToken(token);
        Date created = claims.get(CLAIM_KEY_CREATED, Date.class);
        Date refreshDate = new Date();
        //刷新时间在创建时间的指定时间内
        if (refreshDate.after(created) && refreshDate.before(DateUtil.offsetSecond(created, time))) {
            return true;
        }
        return false;
    }

    public static void main(String[] args) {
        HashMap<String, Object> claims = new HashMap<>();
        claims.put(CLAIM_KEY_CREATED, new Date(System.currentTimeMillis() - 1000L * 60 * 60 * 48));
        claims.put(CLAIM_KEY_APP_ID, "portal");
        claims.put(CLAIM_KEY_USERID, 1);
        String token = Jwts.builder()
                .setClaims(claims)
                .setExpiration(new Date(System.currentTimeMillis() - 1000L * 60 * 60 * 24))
                .signWith(SignatureAlgorithm.HS512, SECRET)
                .compact();
        System.out.println(token);
        boolean expired = isTokenExpired(token);
        System.out.println(expired);
        /*Integer userId = getUserIdFromToken("eyJhbGciOiJIUzUxMiJ9.eyJjcmVhdGVkIjoxNjI3ODAxMzIzMDE5LCJhcHBJZCI6InBvcnRhbCIsImV4cCI6MTYyNzg4NzcyMywidXNlcklkIjoxfQ.xch9RdpHupq64l3I2RvU09E2giCYfn2lfHmv1OYQhVpEVqe9NcjZ2mxcm4_2XapC1AHNGP6TRQI3BB3G4ZB-FA");
        System.out.println(userId);*/
    }
}
