package com.fzm.common.utils;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import com.aliyun.oss.OSS;
import com.aliyun.oss.OSSClientBuilder;
import com.aliyun.oss.common.utils.BinaryUtil;
import com.aliyun.oss.model.*;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.properties.OssProperties;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * @author tangtuo
 * @date 2021/6/29 16:07
 */
@Slf4j
@Component
public class OssUtil {

    @Resource
    private OssProperties ossProperties;

    @Resource
    private HttpServletResponse response;

    /**
     * @param file
     * @return
     */
    public String putObject(MultipartFile file, String bucket) throws IOException {
        String endpoint = ossProperties.getEndPoint();
        // 阿里云主账号AccessKey拥有所有API的访问权限，风险很高。强烈建议您创建并使用RAM账号进行API访问或日常运维，请登录RAM控制台创建RAM账号。
        String accessKeyId = ossProperties.getAppKey();
        String accessKeySecret = ossProperties.getAppSecret();
        // <yourObjectName>上传文件到OSS时需要指定包含文件后缀在内的完整路径，例如20210629/a8703bcf7d354ea2bbb5ccdf76b7405f/123.jpg。
        String objectName = String.format("%s/%s/%s", DateUtil.format(new Date(), "yyyyMMdd"), IdUtil.simpleUUID(), file.getOriginalFilename());

        // 创建OSSClient实例。
        OSS ossClient = new OSSClientBuilder().build(endpoint, accessKeyId, accessKeySecret);
        ossClient.putObject(bucket, objectName, file.getInputStream());

        // 关闭OSSClient。
        ossClient.shutdown();
        return String.format("https://%s.%s/%s", bucket, endpoint, objectName);
    }

    /**
     * 文件普通存储
     *
     * @param file
     * @return
     * @throws IOException
     */
    public String putSimpleObject(MultipartFile file) throws IOException {
        return putObject(file, ossProperties.getSimpleBucket());
    }

    /**
     * 文件加密存储
     *
     * @param file
     * @return
     * @throws IOException
     */
    public String putEncryptObject(MultipartFile file) throws IOException {
        return putObject(file, ossProperties.getEncryptBucket());
    }

    public void createEncryptBucket() {
        String endpoint = ossProperties.getEndPoint();
        // 阿里云主账号AccessKey拥有所有API的访问权限，风险很高。强烈建议您创建并使用RAM账号进行API访问或日常运维，请登录RAM控制台创建RAM账号。
        String accessKeyId = ossProperties.getAppKey();
        String accessKeySecret = ossProperties.getAppSecret();
        OSS ossClient = new OSSClientBuilder().build(endpoint, accessKeyId, accessKeySecret);

        // 创建存储空间。
        String bucket = ossProperties.getEncryptBucket();
        if (!ossClient.doesBucketExist(bucket)) {
            // 设置Bucket加密。
            ServerSideEncryptionByDefault applyServerSideEncryptionByDefault = new ServerSideEncryptionByDefault(SSEAlgorithm.AES256);
            ServerSideEncryptionConfiguration sseConfig = new ServerSideEncryptionConfiguration();
            sseConfig.setApplyServerSideEncryptionByDefault(applyServerSideEncryptionByDefault);
            SetBucketEncryptionRequest request = new SetBucketEncryptionRequest("<yourBucketName>", sseConfig);
            ossClient.setBucketEncryption(request);
            ossClient.shutdown();
            ossClient.createBucket(bucket);
        }
        ossClient.createBucket(ossProperties.getEncryptBucket());
    }


    /**
     * @param fileUrl  文件地址
     * @param fileName 文件名
     */
    public void downloadFile(String fileUrl, String fileName) {
        String endpoint = ossProperties.getEndPoint();
        // 阿里云主账号AccessKey拥有所有API的访问权限，风险很高。强烈建议您创建并使用RAM账号进行API访问或日常运维，请登录RAM控制台创建RAM账号。
        String accessKeyId = ossProperties.getAppKey();
        String accessKeySecret = ossProperties.getAppSecret();
        String bucketName = ossProperties.getEncryptBucket();
        try {
            // 文件名以附件的形式下载
            response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8"));
        } catch (UnsupportedEncodingException e) {
            log.error(e.getMessage(), e);
            throw GlobalException.newException(ResultCode.FILE_DOWNLOAD_ERROR, e.getMessage());
        }
        // 创建OSSClient实例。
        OSS ossClient = new OSSClientBuilder().build(endpoint, accessKeyId, accessKeySecret);
        // ossObject包含文件所在的存储空间名称、文件名称、文件元信息以及一个输入流。
        fileUrl = fileUrl.replace(String.format("https://%s.%s/", bucketName, endpoint), "");
        OSSObject ossObject = ossClient.getObject(bucketName, fileUrl);
        try {
            // 读取文件内容。
            InputStream inputStream = ossObject.getObjectContent();
            BufferedInputStream in = new BufferedInputStream(inputStream);// 把输入流放入缓存流
            ServletOutputStream outputStream = response.getOutputStream();
            BufferedOutputStream out = new BufferedOutputStream(outputStream);// 把输出流放入缓存流
            byte[] buffer = new byte[1024];
            int len = 0;
            while ((len = in.read(buffer)) != -1) {
                out.write(buffer, 0, len);
            }
            if (out != null) {
                out.flush();
                out.close();
            }
            if (in != null) {
                in.close();
            }
            ossClient.shutdown();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw GlobalException.newException(ResultCode.FILE_DOWNLOAD_ERROR, e.getMessage());
        }

    }

    public Map<String, String> policy() {
        String accessId = ossProperties.getAppKey(); // 请填写您的AccessKeyId。
        String accessKey = ossProperties.getAppSecret(); // 请填写您的AccessKeySecret。
        String endpoint = ossProperties.getEndPoint(); // 请填写您的 endpoint。
        String bucket = ossProperties.getEncryptBucket(); // 请填写您的 bucketname 。
        String host = "https://" + bucket + "." + endpoint; // host的格式为 bucketname.endpoint
        String dir = DateUtil.format(new Date(), "yyyyMMdd") + "/" + IdUtil.simpleUUID() + "/";
        Map<String, String> respMap = new LinkedHashMap<>();
        // 创建OSSClient实例。
        OSS ossClient = new OSSClientBuilder().build(endpoint, accessId, accessKey);
        try {
            long expireTime = 120;
            long expireEndTime = System.currentTimeMillis() + expireTime * 1000;
            Date expiration = new Date(expireEndTime);
            // PostObject请求最大可支持的文件大小为5 GB，即CONTENT_LENGTH_RANGE为5*1024*1024*1024。
            PolicyConditions policyConds = new PolicyConditions();
            policyConds.addConditionItem(PolicyConditions.COND_CONTENT_LENGTH_RANGE, 0, 1048576000);
            policyConds.addConditionItem(MatchMode.StartWith, PolicyConditions.COND_KEY, dir);
            String postPolicy = ossClient.generatePostPolicy(expiration, policyConds);
            byte[] binaryData = postPolicy.getBytes("utf-8");
            String encodedPolicy = BinaryUtil.toBase64String(binaryData);
            String postSignature = ossClient.calculatePostSignature(postPolicy);
            respMap.put("accessid", accessId);
            respMap.put("policy", encodedPolicy);
            respMap.put("signature", postSignature);
            respMap.put("dir", dir);
            respMap.put("host", host);
            respMap.put("expire", String.valueOf(expireEndTime / 1000));
            return respMap;
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
            throw GlobalException.newException(ResultCode.FILE_UPLOAD_ERROR, e.getMessage());
        }
    }

    public void delete(String fileUrl, String bucketName) {
        String endpoint = ossProperties.getEndPoint();
        // 阿里云主账号AccessKey拥有所有API的访问权限，风险很高。强烈建议您创建并使用RAM账号进行API访问或日常运维，请登录RAM控制台创建RAM账号。
        String accessKeyId = ossProperties.getAppKey();
        String accessKeySecret = ossProperties.getAppSecret();
        // <yourObjectName>上传文件到OSS时需要指定包含文件后缀在内的完整路径，例如20210629/a8703bcf7d354ea2bbb5ccdf76b7405f/123.jpg。
        String objectName = fileUrl.replace(String.format("https://%s.%s/", bucketName, endpoint), "");
        // 创建OSSClient实例。
        OSS ossClient = new OSSClientBuilder().build(endpoint, accessKeyId, accessKeySecret);
        ossClient.deleteObject(bucketName, objectName);
    }


    public static void main(String[] args) {
        String endpoint = "oss-cn-shanghai.aliyuncs.com";
        // 阿里云主账号AccessKey拥有所有API的访问权限，风险很高。强烈建议您创建并使用RAM账号进行API访问或日常运维，请登录RAM控制台创建RAM账号。
        String accessKeyId = "LTAI5tAAXBCtnQ2ytwagazjt";
        String accessKeySecret = "x0T6hyJvRFLtPjZYrNo5egEc4kShlU";
        OSS ossClient = new OSSClientBuilder().build(endpoint, accessKeyId, accessKeySecret);

        String bucket2 = "joying-nft";


        if (ossClient.doesBucketExist(bucket2)) {
            // 设置Bucket加密。
            ServerSideEncryptionByDefault applyServerSideEncryptionByDefault = new ServerSideEncryptionByDefault(SSEAlgorithm.AES256);
            ServerSideEncryptionConfiguration sseConfig = new ServerSideEncryptionConfiguration();
            sseConfig.setApplyServerSideEncryptionByDefault(applyServerSideEncryptionByDefault);
            SetBucketEncryptionRequest request = new SetBucketEncryptionRequest(bucket2, sseConfig);
            ossClient.setBucketEncryption(request);
            ossClient.shutdown();
        }
    }
}
