package com.fzm.portal.controller;

import cn.hutool.crypto.SecureUtil;
import com.fzm.common.annotation.Authentication;
import com.fzm.common.constant.RedisConstant;
import com.fzm.common.constant.SystemConstant;
import com.fzm.common.constant.TokenConstant;
import com.fzm.common.entity.Nft;
import com.fzm.common.entity.NftDto;
import com.fzm.common.entity.User;
import com.fzm.common.entity.vo.CollectionNftVo;
import com.fzm.common.entity.vo.NftCertificateVo;
import com.fzm.common.entity.vo.NftTransferVo;
import com.fzm.common.entity.vo.NftVo;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.model.ResponseModel;
import com.fzm.common.params.NftTransferParam;
import com.fzm.common.service.CategoryService;
import com.fzm.common.service.NftService;
import com.fzm.common.service.UserService;
import com.fzm.common.utils.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author tangtuo
 * @date 2021/6/30 15:08
 */
@Slf4j
@RestController
@RequestMapping(value = "/nft")
@Api(tags = "nft管理")
public class NftController {

    @Resource
    private NftService nftService;

    @Resource
    private UserService userService;

    @Resource
    private CategoryService categoryService;

    @Resource
    private RedisUtil redisUtil;

    @Resource
    private HttpServletRequest request;

    /**
     * 存证二维码跳转地址
     * todo 修改成动态前端路由
     */
    private static final String PATH = "https://chain.33.cn/document/60";


    @Authentication
    @PostMapping("/save")
    @ApiOperation(value = "nft基本信息保存(基本信息和加密上链两个步骤)")
    public ResponseModel<NftDto> save(@Validated @ModelAttribute Nft nft) {
        if (SystemConstant.BOOLEAN_DATA_TRUE.equals(nft.getIsArchives())) {
            if (StringUtils.isAnyBlank(nft.getFileUrl(), nft.getName()) || nft.getIsGrant() == null) {
                throw GlobalException.newException(ResultCode.VALIDATE_FAILED, "当选择平台存档时,文件名和文件路径必传");
            }
        }
        return ResponseModel.success(nftService.saveNft(nft));
    }


    @Authentication
    @PostMapping("/publish")
    @ApiOperation("发行nft")
    public ResponseModel<Integer> publish(@Validated @RequestBody NftDto nftDto) {
        Integer id = nftService.publish(nftDto);
        return ResponseModel.success(id);
    }

    @GetMapping("/list")
    @ApiOperation(value = "获取nft列表")
    public ResponseModel<List<Nft>> list(@ApiParam(value = "当前页码") @RequestParam Integer pageNum,
                                         @ApiParam(value = "每页记录数") @RequestParam Integer pageSize,
                                         @ApiParam(value = "类目id,查询所有的时候传null") @RequestParam(required = false) Integer categoryId) {

        pageNum = (pageNum - 1) * pageSize;
        List<Nft> list = nftService.list(pageNum, pageSize, categoryId);
        return ResponseModel.success(list);
    }

    @GetMapping("get/{id}")
    @ApiOperation(value = "获取nft详情")
    public ResponseModel<NftVo> getById(@PathVariable Integer id, @RequestHeader(required = false) String Authorization) {
        Nft nft = nftService.getById(id);
        if (nft == null) {
            throw GlobalException.newException(ResultCode.DATA_ERROR, "没找到此nft的详情");
        }
        User user = userService.getUserByWallet(nft.getPublishAddress());
        if (user == null) {
            throw GlobalException.newException(ResultCode.DATA_ERROR, "没找到此nft的发布人的信息");
        }
        NftVo nftVo = new NftVo(nft, user);
        nftVo.setCategory(categoryService.getById(nft.getCategoryId()).getCategoryName());
        if (StringUtils.isNotBlank(Authorization)) {
            String realToken = Authorization.substring(TokenConstant.TOKEN_PREFIX.length());
            if (StringUtils.isNotBlank(realToken) && !"null".equals(realToken)) {
                Integer userId = JwtUtil.getUserIdFromToken(Authorization);
                Boolean collection = redisUtil.sIsMember(RedisConstant.COLLECTION_USER_PREFIX + userId, id.toString());
                nftVo.setCollection(collection);
            }
        }
        return ResponseModel.success(nftVo);
    }

    @Authentication
    @GetMapping("/list/current")
    @ApiOperation(value = "获取我的nft列表")
    public ResponseModel<Map<String, Object>> listCurrent(@ApiParam(value = "类目id,查询全部的时候传null") @RequestParam(required = false) Integer categoryId) throws Exception {
        Integer userId = JwtUtil.getUserIdFromToken(request.getHeader("Authorization"));
        List<CollectionNftVo> list = nftService.listCurrent(categoryId, userId);
        User user = userService.getById(userId);
        // 生成二维码
        HashMap<String, Object> qrCodeMap = new HashMap<>();
        qrCodeMap.put("nickname", user.getNickname());
        qrCodeMap.put("avatar", user.getAvatar());
        qrCodeMap.put("wallet", user.getWallet());
        String qrCode = QRCodeUtil.encode(JsonUtil.toJson(qrCodeMap), null, false);
        HashMap<String, Object> result = new HashMap<>(8);
        result.put("list", list);
        result.put("size", list.size());
        result.put("qrCode", qrCode);
        return ResponseModel.success(result);
    }

    @Authentication
    @GetMapping("/generateNftId")
    @ApiOperation(value = "生成nft编号")
    public ResponseModel<String> generateNftId(@ApiParam(value = "类目id", required = true) @RequestParam Integer categoryId) {
        String nftId = nftService.generateNftId(categoryId);
        return ResponseModel.success(nftId);
    }

    @Authentication
    @PostMapping("/file/md5")
    @ApiOperation(value = "获取文件的md5值")
    public ResponseModel<String> getFileMd5Value(MultipartFile file) throws IOException {
        String md5 = SecureUtil.md5(file.getInputStream());
        return ResponseModel.success(md5);
    }


    @Authentication
    @GetMapping("/certificate/{id}")
    @ApiOperation(value = "nft证书")
    public ResponseModel<NftCertificateVo> certificate(@PathVariable Integer id, @RequestHeader String Authorization) throws Exception {
        Nft nft = nftService.getById(id);
        if (nft == null) {
            throw GlobalException.newException(ResultCode.DATA_ERROR, "没找到此nft的详情");
        }
        int userId = JwtUtil.getUserIdFromToken(Authorization);
        if (!nft.getUserId().equals(userId)) {
            throw GlobalException.newException(ResultCode.FORBIDDEN, "您无权查看别人的nft证书");
        }
        User user = userService.getUserByWallet(nft.getPublishAddress());
        String qrCode = QRCodeUtil.encode(PATH, null, false);
        NftCertificateVo vo = new NftCertificateVo(nft, user, qrCode);
        return ResponseModel.success(vo);
    }

    @Authentication
    @ApiOperation(value = "下载文件")
    @PostMapping("/download/{id}")
    public void download(@ApiParam(value = "nft主键") @PathVariable Integer id) {
        nftService.download(id);
    }

    @Authentication
    @ApiOperation(value = "查询nft转出人的个人信息")
    @GetMapping("/transfer/user")
    public ResponseModel<NftTransferVo> getTransferUserInfo(@RequestParam Integer id) {
        Nft nft = nftService.getById(id);
        if (nft == null) {
            throw GlobalException.newException(ResultCode.DATA_ERROR, "没有找到此nft的详情信息,请稍后再试");
        }
        User user = userService.getById(nft.getUserId());
        return ResponseModel.success(new NftTransferVo(nft, user));
    }

    @Authentication
    @ApiOperation(value = "转让nft")
    @PostMapping("/transfer")
    public ResponseModel<Boolean> transfer(@RequestBody @Validated NftTransferParam param) {
        Boolean result = nftService.transfer(param);
        return ResponseModel.success(result);
    }


}
