package com.fzm.portal.controller;

import com.fzm.common.annotation.Authentication;
import com.fzm.common.constant.RedisConstant;
import com.fzm.common.constant.TokenConstant;
import com.fzm.common.entity.AuthPerson;
import com.fzm.common.entity.User;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.model.ResponseModel;
import com.fzm.common.params.LoginParam;
import com.fzm.common.service.UserService;
import com.fzm.common.utils.JwtUtil;
import com.fzm.common.utils.OssUtil;
import com.fzm.common.utils.RedisUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @author tangtuo
 * @date 2021/6/23 10:59
 */
@RestController
@RequestMapping(value = "/user")
@Api(tags = "用户管理")
public class UserController {

    @Resource
    private UserService userService;

    @Resource
    private OssUtil ossUtil;

    @Resource
    private RedisUtil redisUtil;

    @PostMapping(value = "/login")
    @ApiOperation(value = "登录|注册")
    public ResponseModel<Map<String, Object>> login(@RequestBody @Validated LoginParam param) {
        if (StringUtils.isAllBlank(param.getTelephone(), param.getEmail())) {
            throw GlobalException.newException(ResultCode.VALIDATE_FAILED, "手机号和邮箱不能同时为空");
        }
        if (StringUtils.isNoneBlank(param.getTelephone(), param.getEmail())) {
            throw GlobalException.newException(ResultCode.VALIDATE_FAILED, "手机号和邮箱只能一个为空");
        }
        if (StringUtils.isAllBlank(param.getPassword(), param.getVerificationCode())) {
            throw GlobalException.newException(ResultCode.VALIDATE_FAILED, "密码和验证码不能同事为空");
        }
        if (StringUtils.isNoneBlank(param.getPassword(), param.getVerificationCode())) {
            throw GlobalException.newException(ResultCode.VALIDATE_FAILED, "密码和验证码只能一个为空");
        }
        User user = userService.login(param);
        String token = JwtUtil.generateToken(user);
        redisUtil.setEx(RedisConstant.PORTAL_USER_TOKEN_PREFIX + user.getId(), token, 24L, TimeUnit.HOURS);
        HashMap<String, Object> result = new HashMap<>();
        result.put("user", user);
        result.put(TokenConstant.TOKEN_HEADER, token);
        return ResponseModel.success(result);
    }

    @GetMapping(value = "/isRegister/{username}")
    @ApiOperation(value = "判断用户是否已注册过")
    public ResponseModel<Boolean> isRegister(@PathVariable String username) {
        return ResponseModel.success(userService.loadUserByUsername(username) != null);
    }

    @GetMapping(value = "/isSetPassword/{username}")
    @ApiOperation(value = "判断用户是否已设置密码")
    public ResponseModel<Boolean> isSetPassword(@PathVariable String username) {
        User user = userService.loadUserByUsername(username);
        return ResponseModel.success(user != null && StringUtils.isNotBlank(user.getPassword()));
    }

    @Authentication
    @PostMapping(value = "/logout")
    @ApiOperation(value = "退出登录")
    public ResponseModel<Boolean> logout(@RequestHeader String Authorization) {
        Integer userId = JwtUtil.getUserIdFromToken(Authorization);
        redisUtil.delete(RedisConstant.PORTAL_USER_TOKEN_PREFIX + userId);
        return ResponseModel.success();
    }

    //@ApiOperation(value = "设置密码")
    public ResponseModel<Boolean> setPassword() {
        return ResponseModel.success();
    }

    @Authentication
    @ApiOperation(value = "修改昵称 | 修改个性签名", notes = "用户修改哪项信息就传那一个参数就可以了")
    @PostMapping("/update")
    public ResponseModel<User> update(@Validated @RequestBody User user, @RequestHeader String Authorization) {
        Integer userId = JwtUtil.getUserIdFromToken(Authorization);
        user.setId(userId);
        userService.updateById(user);
        return ResponseModel.success(userService.getById(userId));
    }

    @Authentication
    @ApiOperation(value = "上传头像")
    @PostMapping("/upload/avatar")
    public ResponseModel<User> uploadAvatar(MultipartFile file) throws IOException {
        String avatar = ossUtil.putSimpleObject(file);
        User user = userService.uploadAvatar(avatar);
        return ResponseModel.success(user);
    }

    @Authentication
    @GetMapping("/detail")
    @ApiOperation(value = "获取用户详情")
    public ResponseModel<User> getUserDetail() {
        return ResponseModel.success(userService.getUserByToken());
    }


    @Authentication
    @PostMapping("/auth/person")
    @ApiOperation(value = "个人认证")
    public ResponseModel<Boolean> personAuth(@RequestBody @Validated AuthPerson authPerson) {
        return ResponseModel.success(userService.personAuth(authPerson));
    }


    @Authentication
    @PostMapping("/auth/enterprise")
    @ApiOperation(value = "企业认证")
    public ResponseModel<Boolean> enterpriseAuth() {
        return ResponseModel.success();
    }

    @Authentication
    @ApiOperation("根据钱包地址获取用户详情")
    @GetMapping("/getUserByWallet")
    public ResponseModel<User> getUserByWallet(@RequestParam String wallet) {
        User user = userService.getUserByWallet(wallet);
        return ResponseModel.success(user);
    }
}
